//
// Released into the Public Domain 07/18/91 by Andrew C. Becker
//
// Compile with /n /w
//

#include "set.ch"
#include "setcurs.ch"
#include "box.ch"


FUNCTION Dialog( xText, aChoices, cColor )
  //
  // An ALERT() substitute
  //
  // Author:  Andy Becker  CIS 70523,3153
  //
  // Syntax:  nChoice := Dialog( xText, [aChoices], [cColor] )
  //
  // Where:   xText is either a string or an array of strings
  //          aChoices is an array of strings to serve as prompts
  //            (default is "Ok")
  //          cColor is a color string in Clipper format
  //
  // Returns: Option selected from aChoices, 0 for <Esc>
  //
  LOCAL nRight, nReturn, cScreen, nTop, nLeft, nTextRows, nCol
  LOCAL nTemp, nWidth, nWidth1, nWidth2, nElements, nBottom
  LOCAL bSpace  := SETKEY( 32, { || StuffCR() } )
  LOCAL lWrap   := SET( _SET_WRAP, .t. )
  LOCAL nOldRow := ROW()
  LOCAL nOldCol := COL()
  LOCAL cOldHue := SETCOLOR( cColor )
  LOCAL nCursor := SETCURSOR( SC_NONE )
  IF .NOT. VALTYPE( aChoices ) == 'A'
    aChoices := { "Ok" }                         // default choices
  ENDIF
  nWidth1 := nWidth2 := 0
  IF .NOT. VALTYPE( xText ) == 'A'               // if not an array
    nWidth1 := LEN( xText )                      // assume string
    nTextRows := 1
  ELSE
    nTextRows := LEN( xText )
    FOR nTemp = 1 TO nTextRows                   // get longest string
      nWidth1 := MAX( nWidth1, LEN( xText[nTemp] ) )
    NEXT
  ENDIF
  nElements := LEN( aChoices )
  FOR nTemp = 1 TO nElements                     // find total length of menu options
    nWidth2 += ( LEN( aChoices[nTemp] ) + IIF( nTemp > 1, 4, 2 ) )
  NEXT
  nWidth = MAX( nWidth1 + 6, nWidth2 + 4 )
  // puke if the data won't fit
  IF nWidth + 2 > MAXCOL() .OR. ( MAXROW() / 2 ) - ( nTextRows + 4 ) / 2  + 1 < 0
    xText     := { "A parameter passed to Dialog()", "was too long to display" }
    nTextRows := 2
    nWidth    := 36
    aChoices  := { "Ok" }
    nElements := 1
    nWidth2   := 4
  ENDIF
  // these are for speed and readability (?) later on
  nTop    := ( MAXROW() / 2 ) - ( nTextRows + 4 ) / 2 + 1
  nLeft   := ( ( MAXCOL() + 1 ) / 2 ) - ( nWidth / 2 ) - 1
  nBottom := nTop + nTextRows + 3
  nRight  := nLeft + nWidth + 1
  cScreen := SAVESCREEN( nTop, nLeft, nBottom, nRight )
  DISPBOX( nTop, nLeft, nBottom, nRight, B_SINGLE + CHR( 255 ), cColor )
  // finally display associated text
  IF .NOT. VALTYPE( xText ) == 'A'               // if not an array
    // PADC() doesn't "C" the same way ALERT() does
    @ nTop + 1, nLeft + 1 SAY PADC( " " + xText, nWidth ) COLOR cColor
  ELSE
    FOR nTemp = 1 TO LEN( xText )
      @ nTop + nTemp, nLeft + 1 SAY PADC( " " + xText[nTemp], nWidth ) COLOR cColor
    NEXT
  ENDIF
  // prompt for action
  nCol := ( ( MAXCOL()  + 1 ) / 2 ) - ( nWidth2 / 2 )
  FOR nTemp = 1 TO nElements
    @ nBottom - 1, nCol PROMPT " " + aChoices[nTemp] + " "
    nCol += ( LEN( aChoices[nTemp] ) + 4 )
  NEXT
  MENU TO nReturn
  // cleanup code
  RESTSCREEN( nTop, nLeft, nBottom, nRight, cScreen )
  SETKEY( 32, bSpace )
  SET( _SET_WRAP, lWrap )
  SETCOLOR( cOldHue )
  SETCURSOR( nCursor )
  SETPOS( nOldRow, nOldCol )
RETURN nReturn


STATIC FUNCTION StuffCR()
  KEYBOARD CHR( 13 )
RETURN NIL
